@@ -1,41 +1,51 @@
 #' Differential item functioning tests
 #'
 #' This function runs the Wald and likelihood-ratio approaches for testing differential
-#' item functioning (DIF). This is primarily a convenience wrapper to the 
-#' \code{\link{multipleGroup}} function for performing standard DIF procedures. Models can be 
-#' estimated in parallel automatically by defining a parallel object with \code{\link{mirtCluster}}, 
-#' which will help to decrease the runtime.
+#' item functioning (DIF). This is primarily a convenience wrapper to the
+#' \code{\link{multipleGroup}} function for performing standard DIF procedures. Models can be
+#' estimated in parallel automatically by defining a parallel object with \code{\link{mirtCluster}},
+#' which will help to decrease the runtime. For best results, the baseline model should contain
+#' a set of 'anchor' items and have freely estimated hyper-parameters in the focal groups.
+#'
+#' Generally, the precomputed baseline model should have been
+#' configured with two estimation properties: 1) a set of 'anchor' items exist,
+#' where the anchor items have various parameters that have been constrainted to be equal
+#' across the groups, and 2) contain freely estimated latent mean and variance terms in
+#' all but one group. These two properties help to fix the metric of the groups so that
+#' item parameter estimates do not contain latent distribution characteristics.
 #'
 #' @aliases DIF
-#' @param MGmodel an object returned from \code{\link{multipleGroup}} to be used as the reference 
+#' @param MGmodel an object returned from \code{\link{multipleGroup}} to be used as the reference
 #'   model
-#' @param which.par a character vector containing the parameter names which will be inspected for 
+#' @param which.par a character vector containing the parameter names which will be inspected for
 #'   DIF
 #' @param Wald logical; perform Wald tests for DIF instead of likelihood ratio test?
-#' @param items2test a numeric vector, or character vector containing the item names, indicating 
-#'   which items will be tested for DIF. In models where anchor items are known, omit them from 
-#'   this vector. For example, if items 1 and 2 are anchors in a 10 item test, then 
-#'   \code{items2test = 3:10} would work for testing the remaining items (important to remember 
+#' @param items2test a numeric vector, or character vector containing the item names, indicating
+#'   which items will be tested for DIF. In models where anchor items are known, omit them from
+#'   this vector. For example, if items 1 and 2 are anchors in a 10 item test, then
+#'   \code{items2test = 3:10} would work for testing the remaining items (important to remember
 #'   when using sequential schemes)
+#' @param return_models logical; return estimated model objects for further analysis?
+#'   Default is FALSE
 #' @param scheme type of DIF analysis to perform, either by adding or dropping constraints across
 #'   groups. These can be:
 #' \describe{
-#'   \item{'add'}{parameters in \code{which.par} will be constrained each item one at a time for 
-#'     items that are specified in \code{items2test}. This is beneficial when examining DIF from a 
-#'     model with parameters freely estimated across groups, and when inspecting differences via 
+#'   \item{'add'}{parameters in \code{which.par} will be constrained each item one at a time for
+#'     items that are specified in \code{items2test}. This is beneficial when examining DIF from a
+#'     model with parameters freely estimated across groups, and when inspecting differences via
 #'     the Wald test}
 #'   \item{'drop'}{parameters in \code{which.par} will be freely estimated for items that are
-#'     specified in \code{items2test}. This is useful when supplying an overly restrictive model 
+#'     specified in \code{items2test}. This is useful when supplying an overly restrictive model
 #'     and attempting to detect DIF with a slightly less restrictive model}
-#'   \item{'add_sequential'}{sequentially loop over the items being tested, and at the end of the 
-#'     loop treat DIF tests that satisfy the \code{seq_stat} criteria as invariant. The loop is 
-#'     then re-run on the remaining invariant items to determine if they are now displaying DIF in 
-#'     the less constrained model, and when no new invariant item is found the algorithm stops and 
+#'   \item{'add_sequential'}{sequentially loop over the items being tested, and at the end of the
+#'     loop treat DIF tests that satisfy the \code{seq_stat} criteria as invariant. The loop is
+#'     then re-run on the remaining invariant items to determine if they are now displaying DIF in
+#'     the less constrained model, and when no new invariant item is found the algorithm stops and
 #'     returns the items that displayed DIF}
-#'   \item{'drop_sequential'}{sequentially loop over the items being tested, and at the end of the 
-#'     loop treat items that violate the \code{seq_stat} criteria as demonstrating DIF. The loop is 
-#'     then re-run, leaving the items that previously demonstrated DIF as variable across groups, 
-#'     and the remaining test items that previously showed invariance are re-tested. The algorithm 
+#'   \item{'drop_sequential'}{sequentially loop over the items being tested, and at the end of the
+#'     loop treat items that violate the \code{seq_stat} criteria as demonstrating DIF. The loop is
+#'     then re-run, leaving the items that previously demonstrated DIF as variable across groups,
+#'     and the remaining test items that previously showed invariance are re-tested. The algorithm
 #'     stops when no more items showing DIF are found and returns the items that displayed DIF}
 #' }
 #' @param seq_stat select a statistic to test for in the sequential schemes. Potential values are
@@ -43,12 +53,12 @@
 #'   If a numeric value is input that ranges between 0 and 1, the 'p' value will be tested
 #'   (e.g., \code{seq_stat = .05} will test for the difference of p < .05 in the add scheme,
 #'   or p > .05 in the drop scheme), along with the specified \code{p.adjust} input
-#' @param max_run a number indicating the maximum number of cycles to perform in sequential 
+#' @param max_run a number indicating the maximum number of cycles to perform in sequential
 #'   searches. The default is to perform search until no further DIF is found
 #' @param plotdif logical; create itemplots for items that are displaying DIF according to the
 #'   \code{seq_stat} criteria? Only available for 'add' type schemes
 #' @param type the \code{type} of plot argument passed to \code{plot()}. Default is 'trace', though
-#'   another good option is 'infotrace'. For ease of viewing, the \code{facet_item} argument to 
+#'   another good option is 'infotrace'. For ease of viewing, the \code{facet_item} argument to
 #'   mirt's \code{plot()} function is set to \code{TRUE}
 #' @param p.adjust string to be passed to the \code{\link{p.adjust}} function to adjust p-values.
 #'   Adjustments are located in the \code{adj_pvals} element in the returned list
@@ -56,8 +66,7 @@
 #' @param ... additional arguments to be passed to \code{\link{multipleGroup}} and \code{plot}
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @keywords DIF
-#' @seealso \code{\link{multipleGroup}} 
-#  \code{\link{DTF}}
+#' @seealso \code{\link{multipleGroup}}, \code{\link{DTF}}
 #' @export DIF
 #' @examples
 #' \dontrun{
@@ -80,7 +89,7 @@
 #' # define a parallel cluster (optional) to help speed up internal functions
 #' mirtCluster()
 #'
-#' #  Information matrix with crossprod
+#' #  Information matrix with crossprod (not controlling for latent group differences)
 #' model <- multipleGroup(dat, 1, group, SE = TRUE)
 #'
 #' #test whether adding slopes and intercepts constraints results in DIF. Plot items showing DIF
@@ -126,10 +135,11 @@
 #' stepdown
 #' }
 DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@Data$data),
-                seq_stat = 'SABIC', Wald = FALSE, p.adjust = 'none', max_run = Inf,
-                plotdif = FALSE, type = 'trace', verbose = TRUE, ...){
+                seq_stat = 'SABIC', Wald = FALSE, p.adjust = 'none', return_models = FALSE,
+                max_run = Inf, plotdif = FALSE, type = 'trace', verbose = TRUE, ...){
 
-    loop_test <- function(item, model, which.par, values, Wald, itemnames, invariance, drop, ...)
+    loop_test <- function(item, model, which.par, values, Wald, itemnames, invariance, drop,
+                          return_models, ...)
     {
         constrain <- model@constrain
         parnum <- list()
@@ -140,7 +150,7 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
             if(!length(parnum[[i]])) parnum[[i]] <- NULL
         if(!length(parnum))
             stop('Item ', item, ' does not contain any of the parameters defined in which.par.
-                 Consider removing it from the item2test input or adding relevant parameters 
+                 Consider removing it from the item2test input or adding relevant parameters
                  to which.par')
         if(Wald){
             wv <- wald(model)
@@ -169,6 +179,7 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
                                   verbose = FALSE, ...)
         aov <- anova(newmodel, model, verbose = FALSE)
         attr(aov, 'parnum') <- parnum
+        if(return_models) aov <- newmodel
         return(aov)
     }
 
@@ -178,6 +189,10 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
     itemnames <- colnames(MGmodel@Data$data)
     if(!any(scheme %in% c('add', 'drop', 'add_sequential', 'drop_sequential')))
         stop('scheme input is not valid')
+    if(return_models){
+        if(Wald)
+            stop('return_models argument only valid for likelihood ratio tests')
+    }
     if(Wald){
         if(scheme != 'add')
             stop('Wald tests are only appropriate when add scheme is used')
@@ -196,11 +211,12 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
     invariance <- MGmodel@invariance
     values <- mod2values(MGmodel)
     drop <- scheme == 'drop' || scheme == 'drop_sequential'
-    invariance <- MGmodel@invariance[MGmodel@invariance %in% 
+    invariance <- MGmodel@invariance[MGmodel@invariance %in%
                                          c('free_means', 'free_var', 'free_varcov', 'free_cov')]
     if(!length(invariance)) invariance <- ''
-    res <- myLapply(X=items2test, FUN=loop_test, model=MGmodel, which.par=which.par, values=values, 
-                    Wald=Wald, drop=drop, itemnames=itemnames, invariance=invariance, ...)
+    res <- myLapply(X=items2test, FUN=loop_test, model=MGmodel, which.par=which.par, values=values,
+                    Wald=Wald, drop=drop, itemnames=itemnames, invariance=invariance,
+                    return_models=return_models, ...)
     names(res) <- itemnames[items2test]
     if(scheme %in% c('add_sequential', 'drop_sequential')){
         lastkeep <- rep(TRUE, length(res))
@@ -222,7 +238,7 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
             if(all(keep == lastkeep)) break
             lastkeep <- keep
             if(verbose)
-                cat(sprintf('\rChecking for DIF in %d more items', 
+                cat(sprintf('\rChecking for DIF in %d more items',
                             ifelse(drop, sum(keep), sum(!keep))))
             if(ifelse(drop, sum(keep), sum(!keep)) == 0) break
             constrain <- updatedModel@constrain
@@ -246,7 +262,7 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
                     }
                 }
             }
-            updatedModel <- multipleGroup(MGmodel@Data$data, MGmodel@model[[1L]], 
+            updatedModel <- multipleGroup(MGmodel@Data$data, MGmodel@model[[1L]],
                                           group=MGmodel@Data$group,
                                           invariance = invariance, constrain=constrain,
                                           verbose = FALSE, ...)
@@ -254,7 +270,7 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
             if(drop) pick <- !pick
             tmp <- myLapply(X=items2test[pick], FUN=loop_test, model=updatedModel,
                             which.par=which.par, values=values, Wald=Wald, drop=drop,
-                            itemnames=itemnames, invariance=invariance, ...)
+                            itemnames=itemnames, invariance=invariance, return_models=FALSE, ...)
             names(tmp) <- itemnames[pick]
             for(i in names(tmp))
                 res[[i]] <- tmp[[i]]
@@ -265,12 +281,14 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
             cat('\nComputing final DIF estimates...')
         res <- myLapply(X=items2test[!keep], FUN=loop_test, model=updatedModel,
                         which.par=which.par, values=values, Wald=Wald, drop=FALSE,
-                        itemnames=itemnames, invariance=invariance, ...)
+                        itemnames=itemnames, invariance=invariance, return_models=return_models,
+                        ...)
         names(res) <- itemnames[!keep]
     }
 
     for(i in 1L:length(res))
         attr(res[[i]], 'parnum') <- NULL
+    if(return_models) return(res)
     if(p.adjust != 'none'){
         if(Wald){
             ps <- do.call(c, lapply(res, function(x) x$p))
@@ -309,4 +327,4 @@ DIF <- function(MGmodel, which.par, scheme = 'add', items2test = 1:ncol(MGmodel@
         print(plot(MGmodel, type = type, which.items=which.item, facet_items=TRUE, ...))
     }
     return(res)
-}
\ No newline at end of file
+}
