@@ -1,7 +1,10 @@
-MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(), DERIV)
+MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(),
+                       lrPars = list(), DERIV)
 {
     if(is.null(random)) random <- list()
-    RAND <- length(random) > 0L
+    lr.random <- list() #FIXME overwrite later
+    RAND <- length(random) > 0L; LR.RAND <- length(lr.random) > 0L
+    LRPARS <- length(lrPars) > 0L
     verbose <- list$verbose
     nfact <- list$nfact
     NCYCLES <- list$NCYCLES
@@ -34,6 +37,10 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             estpars <- c(estpars, random[[i]]@est)
         }
     }
+    if(LRPARS){
+        nfullpars <- nfullpars + length(lrPars@par)
+        estpars <- c(estpars, lrPars@est)
+    }
     index <- 1L:nfullpars
     N <- nrow(gtheta0[[1L]])
     #Burn in
@@ -41,7 +48,7 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
     tmp <- .1
     OffTerm <- matrix(0, 1, J)
     for(g in 1L:ngroups){
-        for(i in 1L:30L){
+        for(i in 1L:31L){
             gtheta0[[g]] <- draw.thetas(theta0=gtheta0[[g]], pars=pars[[g]], fulldata=Data$fulldata[[g]],
                                         itemloc=itemloc, cand.t.var=cand.t.var, CUSTOM.IND=CUSTOM.IND,
                                         prior.t.var=gstructgrouppars[[g]]$gcov, OffTerm=OffTerm,
@@ -78,6 +85,8 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             for(i in 1L:length(random))
                 longpars[random[[i]]@parnum] <- random[[i]]@par
         }
+        if(LRPARS)
+            longpars[lrPars@parnum] <- lrPars@par
     }
     names(longpars) <- names(estpars)
     stagecycle <- 1L
@@ -111,6 +120,10 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             UBOUND <- c(UBOUND, random[[i]]@ubound)
         }
     }
+    if(LRPARS){
+        LBOUND <- c(LBOUND, lrPars@lbound)
+        UBOUND <- c(UBOUND, lrPars@ubound)
+    }
     for(g in 1L:ngroups)
         for(i in 1L:J)
             pars[[g]][[i]]@dat <- Data$fulldata[[g]][, c(itemloc[i]:(itemloc[i+1L] - 1L))]
@@ -140,17 +153,27 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
         pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
         for(g in 1L:ngroups)
             gstructgrouppars[[g]] <- ExtractGroupPars(pars[[g]][[J+1L]])
+        if(LRPARS){
+            lrPars@par <- lrPars@beta[] <- longpars[lrPars@parnum]
+            lrPars@mus <- lrPars@X %*% lrPars@beta
+            if(!list$USEEM){
+                tmp <- gtheta0[[1L]] - lrPars@mus
+                lrPars@sigma <- lrPars@sigma + gamma*((t(tmp) %*% tmp)/ nrow(tmp) - lrPars@sigma)
+            }
+            gstructgrouppars[[g]]$gmeans <- lrPars@mus
+        }
         if(RAND && cycles > 100L) random <- reloadRandom(random=random, longpars=longpars,
                                         parstart=max(pars[[1L]][[J+1L]]@parnum) + 1L)
 
         start <- proc.time()[3L]
         if(RAND && cycles == 100L){
-            for(g in 1L:ngroups) gtheta0[[g]] <- matrix(0, nrow(Data$fulldata[[g]]), nfact)
+            gtheta0[[1L]] <- matrix(0, nrow(gtheta0[[1L]]), ncol(gtheta0[[1L]]))
             OffTerm <- OffTerm(random, J=J, N=N)
             for(j in 1L:length(random)){
                 tmp <- .1
-                for(i in 1L:30L){
-                    random[[j]]@drawvals <- DrawValues(random[[j]], Theta=gtheta0[[1L]], itemloc=itemloc,
+                for(i in 1L:31L){
+                    random[[j]]@drawvals <- DrawValues(random[[j]], itemloc=itemloc,
+                                                       Theta=gtheta0[[1L]],
                                                        pars=pars[[1L]], fulldata=Data$fulldata[[1L]],
                                                        offterm0=OffTerm, CUSTOM.IND=CUSTOM.IND)
                     OffTerm <- OffTerm(random, J=J, N=N)
@@ -178,7 +201,7 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             }
             cand.t.var <- .5
             tmp <- .1
-            for(i in 1L:30L){
+            for(i in 1L:31L){
                 gtheta0[[1L]] <- draw.thetas(theta0=gtheta0[[1L]], pars=pars[[1L]], fulldata=Data$fulldata[[1L]],
                                              itemloc=itemloc, cand.t.var=cand.t.var, CUSTOM.IND=CUSTOM.IND,
                                              prior.t.var=gstructgrouppars[[1L]]$gcov, OffTerm=OffTerm,
@@ -208,6 +231,11 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
                 tmp2[pars[[1L]][[length(pars[[1L]])]]@est]
         }
 
+        # deal with latent regression random effects TODO
+        if(LR.RAND){
+            browser()
+        }
+
         #Step 1. Generate m_k datasets of theta
         LL <- 0
         for(g in 1L:ngroups){
@@ -228,6 +256,18 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
                 }
             }
         }
+        #adjust cand.t.var
+        PA <- sapply(gtheta0, function(x) attr(x, "Proportion Accepted"))
+        NS <- sapply(gtheta0, function(x) nrow(x))
+        cand.t.var <- controlCandVar(sum(PA * NS / sum(NS)), cand.t.var)
+        if(RAND && cycles > 101L){
+            for(j in 1:length(random)){
+                random[[j]]@cand.t.var <- controlCandVar(
+                               attr(random[[j]]@drawvals, "Proportion Accepted"),
+                               random[[j]]@cand.t.var, min = .01, max = .5)
+
+            }
+        }
         Draws.time <- Draws.time + proc.time()[3L] - start
 
         #Step 2. Find average of simulated data gradients and hessian
@@ -236,13 +276,13 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
         if(length(prodlist) > 0L)
             gthetatmp <- lapply(gtheta0, function(x, prodlist) prodterms(x, prodlist),
                               prodlist=prodlist)
-        tmp <- .Call('computeDPars', pars, gthetatmp, OffTerm, length(longpars), TRUE, 
+        tmp <- .Call('computeDPars', pars, gthetatmp, OffTerm, length(longpars), TRUE,
                      USE.FIXED)
         g <- tmp$grad; h <- tmp$hess
         if(length(list$SLOW.IND)){
             for(group in 1L:ngroups){
                 for (i in list$SLOW.IND){
-                    deriv <- DERIV[[group]][[i]](x=pars[[group]][[i]], Theta=gthetatmp[[group]], 
+                    deriv <- DERIV[[group]][[i]](x=pars[[group]][[i]], Theta=gthetatmp[[group]],
                                                  estHess=TRUE)
                     g[pars[[group]][[i]]@parnum] <- deriv$grad
                     h[pars[[group]][[i]]@parnum, pars[[group]][[i]]@parnum] <- deriv$hess
@@ -250,8 +290,11 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             }
         }
         for(group in 1L:ngroups){
+            tmptheta <- gtheta0[[group]]
+            if(is(gstructgrouppars[[1L]]$gmeans, 'matrix'))
+                tmptheta <- tmptheta - gstructgrouppars[[1L]]$gmeans
             i <- J + 1L
-            deriv <- Deriv(x=pars[[group]][[i]], Theta=gtheta0[[group]], CUSTOM.IND=CUSTOM.IND)
+            deriv <- Deriv(x=pars[[group]][[i]], Theta=tmptheta, CUSTOM.IND=CUSTOM.IND)
             g[pars[[group]][[i]]@parnum] <- deriv$grad
             h[pars[[group]][[i]]@parnum, pars[[group]][[i]]@parnum] <- deriv$hess
         }
@@ -269,6 +312,15 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
                 }
             }
         }
+        if(LRPARS){
+            inv_sigma <- solve(lrPars@sigma)
+            tmp <- t(inv_sigma %*% t(gtheta0[[1L]] - lrPars@mus) %*% lrPars@X)
+            g[lrPars@parnum] <- as.numeric(tmp)
+            tmp2 <- -det(inv_sigma) * lrPars@tXX
+            for(i in 0:(ncol(tmp)-1))
+                h[lrPars@parnum[1:nrow(tmp) + nrow(tmp)*i],
+                  lrPars@parnum[1:nrow(tmp) + nrow(tmp)*i]] <- tmp2
+        }
         if(length(constrain)){
             grad <- as.numeric(updateGrad(g, L))
             ave.h <- updateHess(-h, L)
@@ -286,21 +338,21 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             AR <- do.call(c, lapply(gtheta0, function(x) attr(x, "Proportion Accepted")))
             CTV <- cand.t.var
             if(RAND && cycles > 100L){
-                AR <- c(AR, do.call(c, lapply(random, 
+                AR <- c(AR, do.call(c, lapply(random,
                                         function(x) attr(x@drawvals, "Proportion Accepted"))))
-                CTV <- c(CTV, do.call(c, lapply(random, 
+                CTV <- c(CTV, do.call(c, lapply(random,
                                                 function(x) x@cand.t.var)))
             }
             AR <- paste0(sapply(AR, function(x) sprintf('%.2f', x)), collapse='; ')
             CTV <- paste0(sapply(CTV, function(x) sprintf('%.2f', x)), collapse='; ')
             if(cycles <= BURNIN)
-                printmsg <- sprintf("\rStage 1 = %i, LL = %.1f, AR(%s) = [%s]", 
+                printmsg <- sprintf("\rStage 1 = %i, LL = %.1f, AR(%s) = [%s]",
                                     cycles, LL, CTV, AR)
             if(cycles > BURNIN && cycles <= BURNIN + SEMCYCLES)
-                printmsg <- sprintf("\rStage 2 = %i, LL = %.1f, AR(%s) = [%s]", 
+                printmsg <- sprintf("\rStage 2 = %i, LL = %.1f, AR(%s) = [%s]",
                                     cycles-BURNIN, LL, CTV, AR)
             if(cycles > BURNIN + SEMCYCLES)
-                printmsg <- sprintf("\rStage 3 = %i, LL = %.1f, AR(%s) = [%s]", 
+                printmsg <- sprintf("\rStage 3 = %i, LL = %.1f, AR(%s) = [%s]",
                                     cycles-BURNIN-SEMCYCLES, LL, CTV, AR)
         }
         if(stagecycle < 3L){
@@ -404,11 +456,14 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             for(g in 1L:ngroups){
                 for(i in 1L:(J+1L))
                     pars[[g]][[i]]@SEpar <- SE[pars[[g]][[i]]@parnum]
-            } 
+            }
             if(RAND){
                 for(i in 1L:length(random))
                     random[[i]]@SEpar <- SE[random[[i]]@parnum]
             }
+            if(LRPARS){
+                lrPars@SEpar <- SE[lrPars@parnum]
+            }
         }
     }
     names(correction) <- names(estpars)[estindex_unique]
@@ -416,7 +471,7 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
     if(!list$SE) info <- matrix(0, 1, 1)
     ret <- list(pars=pars, cycles = cycles - BURNIN - SEMCYCLES, info=if(list$expl) matrix(0) else info,
                 correction=correction, longpars=longpars, converge=converge, SElogLik=0, cand.t.var=cand.t.var, L=L,
-                random=random, time=c(MH_draws = as.numeric(Draws.time), Mstep=as.numeric(Mstep.time)),
+                random=random, lrPars=lrPars, time=c(MH_draws = as.numeric(Draws.time), Mstep=as.numeric(Mstep.time)),
                 estindex_unique=estindex_unique, shortpars=longpars[estpars & !redun_constr],
                 fail_invert_info=fail_invert_info, Prior=vector('list', ngroups), collectLL=NaN)
     ret
