@@ -1,13 +1,13 @@
 #' Imputing plausible data for missing values
 #'
-#' Given an estimated model from any of mirt's model fitting functions and an estimate of the 
-#' latent trait, impute plausible missing data values. Returns the original data in a 
+#' Given an estimated model from any of mirt's model fitting functions and an estimate of the
+#' latent trait, impute plausible missing data values. Returns the original data in a
 #' \code{data.frame} without any NA values.
 #'
 #'
 #' @aliases imputeMissing
 #' @param x an estimated model x from the mirt package
-#' @param Theta a matrix containing the estimates of the latent trait scores 
+#' @param Theta a matrix containing the estimates of the latent trait scores
 #'   (e.g., via \code{\link{fscores}})
 #' @param ... additional arguments to pass
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
@@ -22,7 +22,7 @@
 #' for(i in 1:20)
 #'     dat[NAperson[i], NAitem[i]] <- NA
 #' (mod <- mirt(dat, 1))
-#' scores <- fscores(mod, method = 'MAP', scores.only = TRUE)
+#' scores <- fscores(mod, method = 'MAP', full.scores = TRUE)
 #'
 #' #re-estimate imputed dataset (good to do this multiple times and average over)
 #' fulldata <- imputeMissing(mod, scores)
@@ -30,7 +30,7 @@
 #'
 #' #with multipleGroup
 #' group <- rep(c('group1', 'group2'), each=500)
-#' mod2 <- multipleGroup(dat, 1, group)
+#' mod2 <- multipleGroup(dat, 1, group, TOL=1e-2)
 #' fs <- fscores(mod2, full.scores=TRUE)
 #' fulldata2 <- imputeMissing(mod2, fs)
 #' }
@@ -55,6 +55,14 @@ imputeMissing <- function(x, Theta, ...){
     nfact <- pars[[1L]]@nfact
     if(!is(Theta, 'matrix') || nrow(Theta) != nrow(x@Data$data) || ncol(Theta) != nfact)
         stop('Theta must be a matrix of size N x nfact')
+    if(any(Theta %in% c(Inf, -Inf))){
+        for(i in 1L:ncol(Theta)){
+            tmp <- Theta[,i]
+            tmp[tmp %in% c(-Inf, Inf)] <- NA
+            Theta[Theta[,i] == Inf, i] <- max(tmp, na.rm=TRUE) + .1
+            Theta[Theta[,i] == -Inf, i] <- min(tmp, na.rm=TRUE) - .1
+        }
+    }
     K <- x@K
     J <- length(K)
     data <- x@Data$data
